@@ -9,8 +9,6 @@ from queue import PriorityQueue
 from threading import Thread
 from typing import Any, Optional
 
-from onefuzztypes.events import EventMessage
-
 from .cache import JobFilter, TopCache
 from .signalr import Stream
 from .top_view import render
@@ -33,12 +31,10 @@ class Top:
         self,
         onefuzz: "Onefuzz",
         logger: logging.Logger,
-        show_details: bool,
         job_filter: JobFilter,
     ):
         self.onefuzz = onefuzz
         self.logger = logger
-        self.show_details = show_details
 
         self.cache = TopCache(onefuzz, job_filter)
         self.queue: PriorityQueue = PriorityQueue()
@@ -52,8 +48,7 @@ class Top:
 
     def handler(self, message: Any) -> None:
         for event_raw in message:
-            message = EventMessage.parse_obj(event_raw)
-            self.cache.add_message(message)
+            self.cache.add_message(event_raw)
 
     def setup(self) -> Stream:
         client = Stream(self.onefuzz, self.logger)
@@ -97,7 +92,7 @@ class Top:
         error: Optional[Exception] = None
         try:
             self.logger.info("rendering")
-            render(self.cache, self.show_details)
+            render(self.cache)
             client.stop()
         except Exception as err:
             error = err
