@@ -9,15 +9,13 @@ import zipfile
 from typing import Any, Dict, List, Optional, Tuple
 from uuid import uuid4
 
-from onefuzztypes.enums import OS, ContainerType, TaskState
+from onefuzztypes.enums import OS, ContainerType, JobState, TaskState
 from onefuzztypes.models import Job, NotificationConfig
 from onefuzztypes.primitives import Container, Directory, File
 
 from ..job_templates.job_monitor import JobMonitor
 
 ELF_MAGIC = b"\x7fELF"
-DEFAULT_LINUX_IMAGE = "Canonical:UbuntuServer:18.04-LTS:latest"
-DEFAULT_WINDOWS_IMAGE = "MicrosoftWindowsDesktop:Windows-10:20h2-pro:latest"
 
 
 class StoppedEarly(Exception):
@@ -177,13 +175,6 @@ class JobHelper:
                 self.containers[ContainerType.inputs], Directory(tmp_dir)
             )
 
-    @classmethod
-    def get_image(_cls, platform: OS) -> str:
-        if platform == OS.linux:
-            return DEFAULT_LINUX_IMAGE
-        else:
-            return DEFAULT_WINDOWS_IMAGE
-
     @classmethod
     def get_platform(_cls, target_exe: File) -> OS:
         with open(target_exe, "rb") as handle:
@@ -209,16 +200,17 @@ class JobHelper:
 
     def check_current_job(self) -> Job:
         job = self.onefuzz.jobs.get(self.job.job_id)
-        if job.state in ["stopped", "stopping"]:
+        if job.state in JobState.shutting_down():
             raise StoppedEarly("job unexpectedly stopped early")
 
         errors = []
-        for task in self.onefuzz.tasks.list(job_id=self.job.job_id):
-            if task.state in ["stopped", "stopping"]:
-                if task.error:
-                    errors.append("%s: %s" % (task.config.task.type, task.error))
-                else:
-                    errors.append("%s" % task.config.task.type)
+        for task in self.onefuzz.tasks.list(
+            job_id=self.job.job_id, state=TaskState.shutting_down()
+        ):
+            if task.error:
+                errors.append("%s: %s" % (task.config.task.type, task.error))
+            else:
+                errors.append("%s" % task.config.task.type)
 
         if errors:
             raise StoppedEarly("tasks stopped unexpectedly.\n%s" % "\n".join(errors))
